@@ -51,7 +51,7 @@ if [ "$LUAJIT" == "yes" ]; then
   fi
 
   make && make install PREFIX="$LUA_HOME_DIR"
-  ln -s $HOME/.usr/bin/luajit $HOME/.usr/bin/lua
+  ln -s $LUA_HOME_DIR/bin/luajit $LUA_HOME_DIR/bin/lua
 else
 
   if [ "$LUA" == "lua5.1" ]; then
@@ -61,8 +61,11 @@ else
     curl http://www.lua.org/ftp/lua-5.2.4.tar.gz | tar xz
     cd lua-5.2.4
   elif [ "$LUA" == "lua5.3" ]; then
-    curl http://www.lua.org/ftp/lua-5.3.5.tar.gz | tar xz
-    cd lua-5.3.5
+    curl http://www.lua.org/ftp/lua-5.3.6.tar.gz | tar xz
+    cd lua-5.3.6
+  elif [ "$LUA" == "lua5.4" ]; then
+    curl http://www.lua.org/ftp/lua-5.4.2.tar.gz | tar xz
+    cd lua-5.4.2
   fi
 
   # Build Lua without backwards compatibility for testing
@@ -71,6 +74,8 @@ else
   make INSTALL_TOP="$LUA_HOME_DIR" install
 fi
 
+export PATH=$LUA_HOME_DIR/bin:$PATH
+
 cd $TRAVIS_BUILD_DIR
 
 lua -v
