@@ -56,7 +56,7 @@ return 1;
 static const char* close_flags[] =
 {
   "noclose",  /* #define BIO_NOCLOSE    0x00 */
-  "close",  /* #define BIO_CLOSE    0x01 */
+  "close",    /* #define BIO_CLOSE      0x01 */
   NULL
 };
 
@@ -82,7 +82,31 @@ static LUA_FUNCTION(openssl_bio_new_mem)
     BIO_write(bio, d, l);
   }
 
-  (void)BIO_set_close(bio, BIO_CLOSE);
+  PUSH_OBJECT(bio, "openssl.bio");
+  return 1;
+}
+
+static LUA_FUNCTION(openssl_bio_new_pair)
+{
+  size_t b1 = luaL_optint(L, 1, 0);
+  size_t b2 = luaL_optint(L, 2, b1);
+  BIO *B1 = NULL;
+  BIO *B2 = NULL;
+
+  int ret = BIO_new_bio_pair(&B1, b1, &B2, b2);
+  if (ret==1)
+  {
+    PUSH_OBJECT(B1, "openssl.bio");
+    PUSH_OBJECT(B2, "openssl.bio");
+    return 2;
+  }
+  return openssl_pushresult(L, ret);
+}
+
+static LUA_FUNCTION(openssl_bio_new_null)
+{
+  BIO *bio = BIO_new(BIO_s_null());
+
   PUSH_OBJECT(bio, "openssl.bio");
   return 1;
 }
@@ -179,55 +203,68 @@ make tcp client socket
 @tparam[opt=true] boolean connect default connect immediately
 @treturn bio
 */
+
+/***
+make tcp client socket
+@function connect
+@tparam address table with hostname, ip, port filed
+@tparam[opt=true] boolean connect default connect immediately
+@treturn bio
+*/
 static int openssl_bio_new_connect(lua_State *L)
 {
-  const char *host = luaL_checkstring(L, 1);
-  BIO* bio = BIO_new_connect((char*)host);
+  BIO* bio = NULL;
   int doconn = 1;
+  int ret = 1;
 
-  if (lua_isstring(L, 2))
+  if (lua_isstring(L, 1))
   {
-    if (BIO_set_conn_port(bio, lua_tostring(L, 2)) <= 0)
-    {
-      BIO_free(bio);
-      bio = NULL;
-    }
-    else
-    {
-      doconn = lua_isnone(L, 3) ? doconn : auxiliar_checkboolean(L, 3);
-    }
+    const char *host = luaL_checkstring(L, 1);
+    bio = BIO_new_connect((char*)host);
   }
-  else
-    doconn = auxiliar_checkboolean(L, 2);
-
-  if (bio)
+  else if (lua_istable(L, 1))
   {
-    int ret = 1;
-    if (doconn)
-    {
-      ret = BIO_do_connect(bio);
-    }
+    bio = BIO_new(BIO_s_connect());
 
-    if (ret == 1)
-    {
+    lua_getfield(L, 1, "hostname");
+    if (!lua_isnil(L, -1)) {
+      BIO_set_conn_hostname(bio, (char*) lua_tostring(L, -1));
+    }
+    lua_pop(L, 1);
+
+    lua_getfield(L, 1, "port");
+    if(!lua_isnil(L, -1)) {
+      BIO_set_conn_port(bio, (char *)lua_tostring(L, -1));
+    }
+    lua_pop(L, 1);
+
+    lua_getfield(L, 1, "ip");
+    if (!lua_isnil(L, -1)) {
+      BIO_set_conn_hostname(bio, (char*) lua_tostring(L, -1));
+    }
+    lua_pop(L, 1);
+  } else
+  {
+    bio = BIO_new(BIO_s_connect());
+    doconn = 0;
+  }
+
+  doconn = lua_isnone(L, 2) ? doconn : lua_toboolean(L, 2);
+  if (doconn)
+    ret = BIO_do_connect(bio);
+
+  if (ret == 1)
+  {
       PUSH_OBJECT(bio, "openssl.bio");
       openssl_newvalue(L, bio);
 
       lua_pushboolean(L, 1);
       openssl_valueset(L, bio, "free_all");
       return 1;
-    }
-    else
-    {
-      BIO_free(bio);
-      luaL_error(L, "Error creating connection to remote machine");
-    }
   }
 
-  if (!bio)
-    luaL_error(L, "Error creating connection BIO");
-
-  return 0;
+  BIO_free(bio);
+  return openssl_pushresult(L, ret);
 }
 
 /***
@@ -272,9 +309,11 @@ static LUA_FUNCTION(openssl_bio_new_filter)
   {
   case 0:
     bio = BIO_new(BIO_f_base64());
+    closeflag = luaL_checkoption(L, 2, "noclose", close_flags);
     break;
   case 1:
     bio = BIO_new(BIO_f_buffer());
+    closeflag = luaL_checkoption(L, 2, "noclose", close_flags);
     break;
   case 2:
   {
@@ -283,6 +322,7 @@ static LUA_FUNCTION(openssl_bio_new_filter)
     const char* k = luaL_checklstring(L, 3, &kl);
     const char* v = luaL_checklstring(L, 4, &il);
     int encrypt = auxiliar_checkboolean(L, 5);
+    closeflag = luaL_checkoption(L, 6, "noclose", close_flags);
 
     bio = BIO_new(BIO_f_cipher());
     BIO_set_cipher(bio, c, (const unsigned char*)k, (const unsigned char*)v, encrypt);
@@ -291,9 +331,11 @@ static LUA_FUNCTION(openssl_bio_new_filter)
   case 3:
   {
     const EVP_MD* md = get_digest(L, 2, NULL);
+    closeflag = luaL_checkoption(L, 3, "noclose", close_flags);
     bio = BIO_new(BIO_f_md());
     ret = BIO_set_md(bio, md);
   }
+  break;
   case 4:
   {
     SSL* ssl = CHECK_OBJECT(2, SSL, "openssl.ssl");
@@ -573,7 +615,15 @@ static LUA_FUNCTION(openssl_bio_push)
   BIO* append = CHECK_OBJECT(2, BIO, "openssl.bio");
   bio = BIO_push(bio, append);
   if (bio)
+  {
     lua_pushvalue(L, 1);
+
+    openssl_newvalue(L, bio);
+    lua_pushboolean(L, 1);
+    openssl_valueset(L, bio, "free_all");
+
+    BIO_up_ref(bio);
+  }
   else
     lua_pushnil(L);
   return 1;
@@ -608,18 +658,55 @@ get mem data, only support mem bio object
 */
 static LUA_FUNCTION(openssl_bio_get_mem)
 {
+  BUF_MEM* mem;
   BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
-  if (BIO_method_type(bio) == BIO_TYPE_MEM)
+  int ret = BIO_get_mem_ptr(bio, &mem);
+  if (ret == 1)
   {
-    BUF_MEM* mem;
-    BIO_get_mem_ptr(bio, &mem);
     lua_pushlstring(L, mem->data, mem->length);
-    return 1;
+    return ret;
+  }
+  return openssl_pushresult(L, ret);
+}
+
+static LUA_FUNCTION(openssl_bio_get_md)
+{
+  BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
+
+	bio = BIO_find_type(bio, BIO_TYPE_MD);
+  if (bio)
+  {
+	  EVP_MD *md;
+	  BIO_get_md(bio, &md);
+    PUSH_OBJECT(bio, "openssl.bio");
+    BIO_up_ref(bio);
+    PUSH_OBJECT(md, "openssl.evp_digest");
+    return 2;
   }
-  luaL_error(L, "#1 BIO must be memory type");
   return 0;
 }
 
+static LUA_FUNCTION(openssl_bio_next)
+{
+  BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
+	bio = BIO_next(bio);
+  if (bio)
+  {
+    PUSH_OBJECT(bio, "openssl.bio");
+    BIO_up_ref(bio);
+    return 1;
+  }
+  return 0;
+}
+
+static LUA_FUNCTION(openssl_bio_cipher_status)
+{
+  BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
+  lua_pushboolean(L, BIO_get_cipher_status(bio));
+  return 1;
+}
+
+
 /* network socket */
 /***
 setup ready and accept client connect
@@ -646,13 +733,8 @@ static LUA_FUNCTION(openssl_bio_accept)
       openssl_valueset(L, nb, "free_all");
       return 1;
     }
-    else
-      return openssl_pushresult(L, ret);
   }
-  else
-    luaL_error(L, "BIO_do_accept fail");
-
-  return 0;
+  return openssl_pushresult(L, ret);
 }
 
 /***
@@ -669,7 +751,7 @@ static LUA_FUNCTION(openssl_bio_shutdown)
   }
   else if (BIO_method_type(bio) & (BIO_TYPE_SOCKET | BIO_TYPE_FD))
   {
-    (void)BIO_shutdown_wr(bio);;
+    (void)BIO_shutdown_wr(bio);
   }
   else
     luaL_error(L, "don't know how to shutdown");
@@ -734,21 +816,62 @@ set fd of bio object
 static LUA_FUNCTION(openssl_bio_fd)
 {
   BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
-  int typ = BIO_method_type(bio);
-  if (typ & BIO_TYPE_FD)
+  int type = BIO_method_type(bio);
+  luaL_argcheck(L,
+                type & (BIO_TYPE_FD
+                       |BIO_TYPE_CONNECT
+                       |BIO_TYPE_ACCEPT
+                       |BIO_TYPE_DGRAM
+                       |BIO_TYPE_SOCKET),
+                1,
+                "not a supported BIO type");
+  if (!lua_isnone(L, 2))
   {
-    int fd = -1;
-    if (!lua_isnone(L, 2))
-    {
-      fd = lua_tointeger(L, 2);
-      BIO_set_fd(bio, fd, BIO_NOCLOSE);
-    }
-    else
-      fd = BIO_get_fd(bio, 0);
-    lua_pushinteger(L, fd);
+    int fd = luaL_checkint(L, 2);
+    BIO_set_fd(bio, fd, BIO_NOCLOSE);
   }
-  else
-    luaL_error(L, "BIO type miss match");
+  lua_pushnumber(L, BIO_get_fd(bio, 0));
+  return 1;
+}
+
+/* BIO_s_file() */
+/*
+# define BIO_set_fp(b,fp,c)      BIO_ctrl(b,BIO_C_SET_FILE_PTR,c,(char *)fp)
+# define BIO_get_fp(b,fpp)       BIO_ctrl(b,BIO_C_GET_FILE_PTR,0,(char *)fpp)
+*/
+
+static LUA_FUNCTION(openssl_bio_seek)
+{
+  BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
+  int type = BIO_method_type(bio);
+  int ofs, ret;
+  luaL_argcheck(L,
+                type & (BIO_TYPE_FD
+                       |BIO_TYPE_FILE),
+                1,
+                "not a fd or file BIO type");
+  ofs = luaL_checkint(L, 2);
+  ret = BIO_seek(bio, ofs);
+  if (ret<0)
+    return openssl_pushresult(L, ret);
+  lua_pushinteger(L, ret);
+  return 1;
+}
+
+static LUA_FUNCTION(openssl_bio_tell)
+{
+  BIO* bio = CHECK_OBJECT(1, BIO, "openssl.bio");
+  int type = BIO_method_type(bio);
+  int ret;
+  luaL_argcheck(L,
+                type & (BIO_TYPE_FD
+                       |BIO_TYPE_FILE),
+                1,
+                "not a fd or file BIO type");
+  ret = BIO_tell(bio);
+  if (ret<0)
+    return openssl_pushresult(L, ret);
+  lua_pushinteger(L, ret);
   return 1;
 }
 
@@ -876,57 +999,69 @@ close bio
 static luaL_Reg bio_funs[] =
 {
   /* generate operation */
-  {"read",  openssl_bio_read  },
-  {"gets",  openssl_bio_gets  },
-  {"write", openssl_bio_write },
-  {"puts",  openssl_bio_puts  },
-  {"flush", openssl_bio_flush },
-  {"close", openssl_bio_free  },
-  {"type",  openssl_bio_type  },
-  {"nbio",  openssl_bio_nbio  },
-  {"reset", openssl_bio_reset },
-  {"retry", openssl_bio_retry },
-  {"pending", openssl_bio_pending },
+  {"read",  openssl_bio_read},
+  {"gets",  openssl_bio_gets},
+  {"write", openssl_bio_write},
+  {"puts",  openssl_bio_puts},
+  {"flush", openssl_bio_flush},
+  {"close", openssl_bio_free},
+  {"type",  openssl_bio_type},
+  {"nbio",  openssl_bio_nbio},
+  {"reset", openssl_bio_reset},
+  {"retry", openssl_bio_retry},
+  {"pending", openssl_bio_pending},
 #if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-  {"set_callback", openssl_bio_set_callback },
+  {"set_callback", openssl_bio_set_callback},
 #endif
 
   /* for filter bio */
-  {"push",  openssl_bio_push  },
-  {"pop",   openssl_bio_pop   },
-  {"free",    openssl_bio_free},
+  {"push",  openssl_bio_push},
+  {"pop",   openssl_bio_pop},
+  {"next",   openssl_bio_next},
+  {"get_md", openssl_bio_get_md},
+  {"cipher_status", openssl_bio_cipher_status},
+  {"free",   openssl_bio_free},
 
   /* for mem */
-  {"get_mem", openssl_bio_get_mem },
+  {"get_mem", openssl_bio_get_mem},
 
   /* network socket */
-  {"accept",    openssl_bio_accept },
-  {"connect",   openssl_bio_connect },
-  {"handshake", openssl_bio_handshake },
+  {"accept",    openssl_bio_accept},
+  {"connect",   openssl_bio_connect},
+  {"handshake", openssl_bio_handshake},
 
   {"shutdown",  openssl_bio_shutdown},
-  {"fd",        openssl_bio_fd },
+
+  /* BIO_s_datagram(), BIO_s_fd(), BIO_s_socket(),
+   * BIO_s_accept() and BIO_s_connect() */
+  {"fd",        openssl_bio_fd},
+
   {"ssl",       openssl_bio_get_ssl},
 
-  {"__tostring",  auxiliar_tostring },
-  {"__gc",  openssl_bio_free  },
+  /* BIO_s_fd() and BIO_s_file() */
+  {"seek",      openssl_bio_seek},
+  {"tell",      openssl_bio_tell},
+
+  {"__tostring",  auxiliar_tostring},
+  {"__gc",  openssl_bio_free},
 
   {NULL,    NULL}
 };
 
 static luaL_Reg R[] =
 {
-  {"mem",     openssl_bio_new_mem    },
-  {"socket",  openssl_bio_new_socket   },
-  {"dgram",   openssl_bio_new_dgram    },
-  {"fd",      openssl_bio_new_fd     },
-  {"file",    openssl_bio_new_file   },
-  {"filter",  openssl_bio_new_filter   },
+  {"null",    openssl_bio_new_null},
+  {"mem",     openssl_bio_new_mem},
+  {"pair",    openssl_bio_new_pair},
+  {"socket",  openssl_bio_new_socket},
+  {"dgram",   openssl_bio_new_dgram},
+  {"fd",      openssl_bio_new_fd},
+  {"file",    openssl_bio_new_file},
+  {"filter",  openssl_bio_new_filter},
 
-  {"accept",    openssl_bio_new_accept },
+  {"accept",    openssl_bio_new_accept},
   {"connect",   openssl_bio_new_connect},
 
-  {"__call",    openssl_bio_new_mem},
   {NULL,    NULL}
 };
 
