@@ -8,6 +8,9 @@ ec module for lua-openssl binding
 
 #ifndef OPENSSL_NO_EC
 
+static int openssl_push_group_asn1_flag(lua_State *L, int flag);
+static int openssl_push_point_conversion_form(lua_State *L, point_conversion_form_t form);
+
 static int openssl_ecpoint_affine_coordinates(lua_State *L)
 {
   EC_GROUP* g = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
@@ -17,7 +20,7 @@ static int openssl_ecpoint_affine_coordinates(lua_State *L)
   {
     BIGNUM* x = BN_new();
     BIGNUM* y = BN_new();
-    if (EC_POINT_get_affine_coordinates_GFp(g, p, x, y, NULL) == 1)
+    if (EC_POINT_get_affine_coordinates(g, p, x, y, NULL) == 1)
     {
       PUSH_BN(x);
       PUSH_BN(y);
@@ -28,9 +31,9 @@ static int openssl_ecpoint_affine_coordinates(lua_State *L)
   {
     BIGNUM* x = CHECK_OBJECT(3, BIGNUM, "openssl.bn");
     BIGNUM* y = CHECK_OBJECT(4, BIGNUM, "openssl.bn");
-    ret = EC_POINT_set_affine_coordinates_GFp(g, p, x, y, NULL);
+    ret = EC_POINT_set_affine_coordinates(g, p, x, y, NULL);
     if (ret == 0)
-      luaL_error(L, "EC_POINT_set_affine_coordinates_GFp fail");
+      luaL_error(L, "EC_POINT_set_affine_coordinates fail");
     ret = 0;
   }
   return ret;
@@ -73,17 +76,21 @@ static int openssl_ec_group_parse(lua_State*L)
   EC_GROUP_get_cofactor(group, cofactor, ctx);
   AUXILIAR_SETOBJECT(L, cofactor, "openssl.bn", -1, "cofactor");
 
-  AUXILIAR_SET(L, -1, "asn1_flag", EC_GROUP_get_asn1_flag(group), integer);
+  openssl_push_group_asn1_flag(L, EC_GROUP_get_asn1_flag(group));
+  lua_setfield(L, -2, "asn1_flag");
+
   AUXILIAR_SET(L, -1, "degree", EC_GROUP_get_degree(group), integer);
   AUXILIAR_SET(L, -1, "curve_name", EC_GROUP_get_curve_name(group), integer);
-  AUXILIAR_SET(L, -1, "conversion_form", EC_GROUP_get_point_conversion_form(group), integer);
+
+  openssl_push_point_conversion_form(L, EC_GROUP_get_point_conversion_form(group));
+  lua_setfield(L, -2, "conversion_form");
 
   AUXILIAR_SETLSTR(L, -1, "seed", EC_GROUP_get0_seed(group), EC_GROUP_get_seed_len(group));
 
   a = BN_new();
   b = BN_new();
   p = BN_new();
-  EC_GROUP_get_curve_GFp(group, p, a, b, ctx);
+  EC_GROUP_get_curve(group, p, a, b, ctx);
   lua_newtable(L);
   {
     AUXILIAR_SETOBJECT(L, p, "openssl.bn", -1, "p");
@@ -102,44 +109,55 @@ static int openssl_ec_group_free(lua_State*L)
   return 0;
 }
 
+static int openssl_to_group_asn1_flag(lua_State *L, int i, const char* defval)
+{
+  const char* const flag[] = {"explicit", "named_curve",  NULL};
+  int f = luaL_checkoption(L, i, defval, flag);
+  int form = 0;
+  if (f == 0)
+    form = 0;
+  else if (f == 1)
+    form = OPENSSL_EC_NAMED_CURVE;
+  else
+    luaL_argerror(L, i, "invalid paramater, only accept 'explicit' or 'named_curve'");
+  return form;
+}
+
+static int openssl_push_group_asn1_flag(lua_State *L, int flag)
+{
+  if (flag==0)
+    lua_pushstring(L, "explicit");
+  else if(flag == 1)
+    lua_pushstring(L, "named_curve");
+  else
+    lua_pushnil(L);
+  return 1;
+}
+
 static int openssl_ec_group_asn1_flag(lua_State*L)
 {
   EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
-  int asn1_flag = 0;
+  int asn1_flag;
   if (lua_isnone(L, 2))
   {
     asn1_flag = EC_GROUP_get_asn1_flag(group);
-    if (asn1_flag == 0)
-      lua_pushstring(L, "explicit");
-    else if (asn1_flag == OPENSSL_EC_NAMED_CURVE)
-      lua_pushstring(L, "named_curve");
-    else
-      lua_pushnil(L);
+    openssl_push_group_asn1_flag(L, asn1_flag);
     lua_pushinteger(L, asn1_flag);
     return 2;
   }
-  else if (lua_isstring(L, 2))
-  {
-    /* OPENSSL_EC_NAMED_CURVE,   0 */
-    const char* const options[] = {"named_curve", "explicit", NULL};
-    asn1_flag = luaL_checkoption(L, 2, NULL, options);
-    EC_GROUP_set_asn1_flag(group, asn1_flag);
-  }
   else if (lua_isnumber(L, 2))
-  {
     asn1_flag = luaL_checkint(L, 2);
-    EC_GROUP_set_asn1_flag(group, asn1_flag);
-  }
   else
-    luaL_argerror(L, 2, "not accept type of asn1 flag");
-
-  return 0;
+    asn1_flag = openssl_to_group_asn1_flag(L, 2, NULL);
+  EC_GROUP_set_asn1_flag(group, asn1_flag);
+  lua_pushvalue(L, 1);
+  return 1;
 }
 
-static point_conversion_form_t openssl_point_conversion_form(lua_State *L, int i, const char* defval)
+static point_conversion_form_t openssl_to_point_conversion_form(lua_State *L, int i, const char* defval)
 {
   const char* options[] = {"compressed", "uncompressed", "hybrid", NULL};
-  int f = luaL_checkoption(L, 2, defval, options);
+  int f = luaL_checkoption(L, i, defval, options);
   point_conversion_form_t form = 0;
   if (f == 0)
     form = POINT_CONVERSION_COMPRESSED;
@@ -148,44 +166,43 @@ static point_conversion_form_t openssl_point_conversion_form(lua_State *L, int i
   else if (f == 2)
     form = POINT_CONVERSION_HYBRID;
   else
-    luaL_argerror(L, i, "not accept value point_conversion_form");
+    luaL_argerror(L, i, "invalid paramater, only support 'compressed', 'uncompressed' or 'hybrid'");
   return form;
 }
 
+static int openssl_push_point_conversion_form(lua_State *L, point_conversion_form_t form)
+{
+  if (form == POINT_CONVERSION_COMPRESSED)
+    lua_pushstring(L, "compressed");
+  else if (form == POINT_CONVERSION_UNCOMPRESSED)
+    lua_pushstring(L, "uncompressed");
+  else if (form == POINT_CONVERSION_HYBRID)
+    lua_pushstring(L, "hybrid");
+  else
+    lua_pushnil(L);
+  return 1;
+}
+
 static int openssl_ec_group_point_conversion_form(lua_State*L)
 {
   EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
-  point_conversion_form_t form = 0;
+  point_conversion_form_t form;
   if (lua_isnone(L, 2))
   {
     form = EC_GROUP_get_point_conversion_form(group);
-    if (form == POINT_CONVERSION_COMPRESSED)
-      lua_pushstring(L, "compressed");
-    else if (form == POINT_CONVERSION_UNCOMPRESSED)
-      lua_pushstring(L, "uncompressed");
-    else if (form == POINT_CONVERSION_HYBRID)
-      lua_pushstring(L, "hybrid");
-    else
-      lua_pushnil(L);
+    openssl_push_point_conversion_form(L, form);
     lua_pushinteger(L, form);
     return 2;
   }
-  else if (lua_isstring(L, 2))
-  {
-    form = openssl_point_conversion_form(L, 2, NULL);
-    EC_GROUP_set_point_conversion_form(group, form);
-  }
   else if (lua_isnumber(L, 2))
-  {
     form = luaL_checkint(L, 2);
-    EC_GROUP_set_point_conversion_form(group, form);
-  }
   else
-    luaL_argerror(L, 2, "not accept type of point_conversion_form");
-  return 0;
+    form = openssl_to_point_conversion_form(L, 2, NULL);
+  EC_GROUP_set_point_conversion_form(group, form);
+  lua_pushvalue(L, 1);
+  return 1;
 }
 
-
 EC_GROUP* openssl_get_ec_group(lua_State* L, int ec_name_idx, int param_enc_idx,
                                int conv_form_idx)
 {
@@ -229,16 +246,7 @@ EC_GROUP* openssl_get_ec_group(lua_State* L, int ec_name_idx, int param_enc_idx,
       int form = 0;
       if (lua_isstring(L, param_enc_idx))
       {
-        const char* options[] = {"compressed", "uncompressed", "hybrid", NULL};
-        int f = luaL_checkoption(L, param_enc_idx, NULL, options);
-        if (f == 0)
-          form = POINT_CONVERSION_COMPRESSED;
-        else if (f == 1)
-          form = POINT_CONVERSION_UNCOMPRESSED;
-        else if (f == 2)
-          form = POINT_CONVERSION_HYBRID;
-        else
-          luaL_argerror(L, param_enc_idx, "not accept value point_conversion_form");
+        form = openssl_to_point_conversion_form(L, param_enc_idx, NULL);
         EC_GROUP_set_point_conversion_form(g, form);
       }
       else if (lua_isnumber(L, param_enc_idx))
@@ -261,9 +269,7 @@ EC_GROUP* openssl_get_ec_group(lua_State* L, int ec_name_idx, int param_enc_idx,
       int asn1_flag = 0;
       if (lua_isstring(L, conv_form_idx))
       {
-        /* OPENSSL_EC_NAMED_CURVE,   0 */
-        const char* const options[] = {"named_curve", "explicit", NULL};
-        asn1_flag = luaL_checkoption(L, conv_form_idx, NULL, options);
+        asn1_flag =  openssl_to_group_asn1_flag(L, conv_form_idx, NULL);
         EC_GROUP_set_asn1_flag(g, asn1_flag);
       }
       else if (lua_isnumber(L, conv_form_idx))
@@ -303,6 +309,17 @@ static int openssl_ec_point_dup(lua_State *L)
   return 1;
 }
 
+static int openssl_ec_point_equal(lua_State *L)
+{
+  const EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
+  const EC_POINT* a = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
+  const EC_POINT* b = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
+
+  int ret = EC_POINT_cmp(group, a, b, NULL)==0;
+  lua_pushboolean(L, ret);
+  return 1;
+}
+
 static int openssl_ec_point_oct2point(lua_State *L)
 {
   const EC_GROUP* group  = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
@@ -322,7 +339,9 @@ static int openssl_ec_point_point2oct(lua_State *L)
 {
   const EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
   const EC_POINT* point = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
-  point_conversion_form_t form = openssl_point_conversion_form(L, 3, "uncompressed");
+  point_conversion_form_t form = lua_isnone(L, 3) ?
+                                 EC_GROUP_get_point_conversion_form(group) :
+                                 openssl_to_point_conversion_form(L, 3, "uncompressed");
   size_t size = EC_POINT_point2oct(group, point, form, NULL, 0, NULL);
   if(size>0)
   {
@@ -354,8 +373,9 @@ static int openssl_ec_point_point2bn(lua_State *L)
 {
   const EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
   const EC_POINT* point = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
-  point_conversion_form_t form = openssl_point_conversion_form(L, 3, "uncompressed");
-
+  point_conversion_form_t form = lua_isnone(L, 3) ?
+                                 EC_GROUP_get_point_conversion_form(group) :
+                                 openssl_to_point_conversion_form(L, 3, "uncompressed");
   BIGNUM *bn = EC_POINT_point2bn(group, point, form, NULL, NULL);
   if(bn)
     PUSH_OBJECT(bn, "openssl.bn");
@@ -381,8 +401,9 @@ static int openssl_ec_point_point2hex(lua_State *L)
 {
   const EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
   const EC_POINT* point = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
-  point_conversion_form_t form = openssl_point_conversion_form(L, 3, "uncompressed");
-
+  point_conversion_form_t form = lua_isnone(L, 3) ?
+                                 EC_GROUP_get_point_conversion_form(group) :
+                                 openssl_to_point_conversion_form(L, 3, "uncompressed");
   char* hex = EC_POINT_point2hex(group, point, form, NULL);
   if(hex)
   {
@@ -394,11 +415,41 @@ static int openssl_ec_point_point2hex(lua_State *L)
   return 1;
 }
 
+static int openssl_ec_group_generate_key(lua_State *L)
+{
+  const EC_GROUP* group = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
+
+  EC_KEY *ec = EC_KEY_new();
+  if (ec)
+  {
+    int ret;
+    EC_KEY_set_group(ec, group);
+    ret = EC_KEY_generate_key(ec);
+    if (ret==1)
+    {
+      PUSH_OBJECT(ec, "openssl.ec_key");
+      return 1;
+    }
+    EC_KEY_free(ec);
+    return openssl_pushresult(L, ret);
+  }
+  return 0;
+}
+
+static int openssl_ec_group_equal(lua_State *L)
+{
+  const EC_GROUP* a = CHECK_OBJECT(1, EC_GROUP, "openssl.ec_group");
+  const EC_GROUP* b = CHECK_OBJECT(2, EC_GROUP, "openssl.ec_group");
+  lua_pushboolean(L, EC_GROUP_cmp(a, b, NULL)==0);
+  return 1;
+}
 
 static luaL_Reg ec_group_funs[] =
 {
   {"__tostring",            auxiliar_tostring},
   {"__gc",                  openssl_ec_group_free},
+  {"__eq",                  openssl_ec_group_equal},
+  {"equal",                 openssl_ec_group_equal},
 
   {"affine_coordinates",    openssl_ecpoint_affine_coordinates},
   {"parse",                 openssl_ec_group_parse},
@@ -407,6 +458,7 @@ static luaL_Reg ec_group_funs[] =
   {"point_conversion_form", openssl_ec_group_point_conversion_form},
   {"point_new",             openssl_ec_group_point_new},
   {"point_dup",             openssl_ec_point_dup},
+  {"point_equal",           openssl_ec_point_equal},
   {"point2oct",             openssl_ec_point_point2oct},
   {"oct2point",             openssl_ec_point_oct2point},
   {"point2bn",              openssl_ec_point_point2bn},
@@ -414,10 +466,11 @@ static luaL_Reg ec_group_funs[] =
   {"point2hex",             openssl_ec_point_point2hex},
   {"hex2point",             openssl_ec_point_hex2point},
 
+  {"generate_key",          openssl_ec_group_generate_key},
+
   { NULL, NULL }
 };
 
-
 static int openssl_ecdsa_sign(lua_State*L)
 {
   EC_KEY* ec = CHECK_OBJECT(1, EC_KEY, "openssl.ec_key");
@@ -499,8 +552,8 @@ static int openssl_ecdsa_verify(lua_State*L)
 static int openssl_ec_point_copy(lua_State *L)
 {
   EC_POINT* self = CHECK_OBJECT(1, EC_POINT, "openssl.ec_point");
-  EC_POINT* to = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
-  int ret = EC_POINT_copy(to, self);
+  EC_POINT* from = CHECK_OBJECT(2, EC_POINT, "openssl.ec_point");
+  int ret = EC_POINT_copy(self, from);
   return openssl_pushresult(L, ret);
 }
 
@@ -538,7 +591,7 @@ static int openssl_ec_key_parse(lua_State*L)
     priv = BN_dup(priv);
     AUXILIAR_SETOBJECT(L, priv, "openssl.bn", -1, "d");
 
-    if (EC_POINT_get_affine_coordinates_GFp(group, point, x, y, NULL) == 1)
+    if (EC_POINT_get_affine_coordinates(group, point, x, y, NULL) == 1)
     {
       AUXILIAR_SETOBJECT(L, x, "openssl.bn", -1, "x");
       AUXILIAR_SETOBJECT(L, y, "openssl.bn", -1, "y");
@@ -630,6 +683,13 @@ static int openssl_ecdsa_set_method(lua_State *L)
   return 0;
 }
 
+static int openssl_ec_key_check_key(lua_State *L)
+{
+  EC_KEY *ec = CHECK_OBJECT(1, EC_KEY, "openssl.ec_key");
+  lua_pushboolean(L, EC_KEY_check_key(ec));
+  return 1;
+}
+
 static int openssl_ec_key_export(lua_State *L)
 {
   EC_KEY *ec = CHECK_OBJECT(1, EC_KEY, "openssl.ec_key");
@@ -647,7 +707,7 @@ static int openssl_ec_key_export(lua_State *L)
 static int openssl_ec_key_group(lua_State *L)
 {
   EC_KEY *ec = CHECK_OBJECT(1, EC_KEY, "openssl.ec_key");
-  if(lua_isnone(L, 2))
+  if(!lua_isnone(L, 2))
   {
     EC_GROUP *g = CHECK_OBJECT(2, EC_GROUP, "openssl.ec_group");
     int ret = EC_KEY_set_group(ec, g);
@@ -675,12 +735,53 @@ static int openssl_ec_key_read(lua_State *L)
   return 1;
 }
 
+static int openssl_ec_key_conv_form(lua_State *L)
+{
+  EC_KEY *ec = CHECK_OBJECT(1, EC_KEY, "openssl.ec_key");
+  point_conversion_form_t cform;
+  if (lua_isnone(L, 2))
+  {
+    cform = EC_KEY_get_conv_form(ec);
+    openssl_push_point_conversion_form(L, cform);
+    lua_pushinteger(L, cform);
+    return 2;
+  }
+  else if(lua_isnumber(L, 2))
+    cform = lua_tointeger(L, 2);
+  else
+    cform = openssl_to_point_conversion_form(L, 2, NULL);
+  EC_KEY_set_conv_form(ec, cform);
+  lua_pushvalue(L, 1);
+  return 1;
+}
+
+static int openssl_ec_key_enc_flags(lua_State *L)
+{
+  EC_KEY *ec = CHECK_OBJECT(1, EC_KEY, "openssl.ec_key");
+  unsigned int flags;
+  if (lua_isnone(L, 2))
+  {
+    flags = EC_KEY_get_enc_flags(ec);
+    openssl_push_group_asn1_flag(L, flags);
+    lua_pushinteger(L, flags);
+    return 2;
+  }
+  else if(lua_isnumber(L, 2))
+    flags = luaL_checkint(L, 2);
+  else
+    flags = openssl_to_group_asn1_flag(L, 2, NULL);
+  EC_KEY_set_enc_flags(ec, flags);
+  lua_pushvalue(L, 1);
+  return 1;
+}
+
 #ifdef EC_EXT
 EC_EXT_DEFINE
 #endif
 
 static luaL_Reg ec_key_funs[] =
 {
+  {"check",       openssl_ec_key_check_key},
   {"export",      openssl_ec_key_export},
   {"parse",       openssl_ec_key_parse},
   {"group",       openssl_ec_key_group},
@@ -688,6 +789,8 @@ static luaL_Reg ec_key_funs[] =
   {"verify",      openssl_ecdsa_verify},
   {"compute_key", openssl_ecdh_compute_key},
   {"set_method",  openssl_ecdsa_set_method},
+  {"conv_form",   openssl_ec_key_conv_form},
+  {"enc_flags",   openssl_ec_key_enc_flags},
 
 #ifdef EC_EXT
   EC_EXT
