@@ -10,6 +10,7 @@ create and manage x509 certificate sign request
 #include <stdint.h>
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
 #include <openssl/ts.h>
+#include <openssl/asn1.h>
 
 /***
 create a new ts_req object.
@@ -106,7 +107,7 @@ static LUA_FUNCTION(openssl_ts_resp_ctx_new)
   {
     ret = X509_check_private_key(signer, pkey);
     if (ret != 1)
-      luaL_error(L, "singer cert and private key not match");
+      luaL_error(L, "signer cert and private key not match");
   }
 
   ctx = TS_RESP_CTX_new();
@@ -247,17 +248,7 @@ static int openssl_ts_req_nonce(lua_State*L)
   TS_REQ* req = CHECK_OBJECT(1, TS_REQ, "openssl.ts_req");
   if (lua_isnone(L, 2))
   {
-    const ASN1_INTEGER* ai = TS_REQ_get_nonce(req);
-    if (ai)
-    {
-      BIGNUM *bn;
-      PUSH_ASN1_INTEGER(L, ai);
-      bn = ASN1_INTEGER_to_BN(ai, NULL);
-      PUSH_OBJECT(bn, "openssl.bn");
-      return 2;
-    }
-    lua_pushnil(L);
-    return 0;
+    return openssl_push_asn1integer_as_bn(L, TS_REQ_get_nonce(req));
   }
   else
   {
@@ -420,6 +411,18 @@ static LUA_FUNCTION(openssl_ts_req_export)
   return 0;
 }
 
+static LUA_FUNCTION(openssl_ts_req_add_ext)
+{
+  TS_REQ *ts_req = CHECK_OBJECT(1, TS_REQ, "openssl.ts_req");
+  X509_EXTENSION *x = CHECK_OBJECT(2, X509_EXTENSION, "openssl.x509_extension");
+  int loc = luaL_optint(L, 3, TS_REQ_get_ext_count(ts_req));
+  int ret;
+
+  x = X509_EXTENSION_dup(x);
+  ret = TS_REQ_add_ext(ts_req, x, loc);
+  return openssl_pushresult(L, ret);
+}
+
 /***
 get info as table
 @function info
@@ -430,17 +433,7 @@ static LUA_FUNCTION(openssl_ts_req_info)
   TS_REQ *req = CHECK_OBJECT(1, TS_REQ, "openssl.ts_req");
 
   lua_newtable(L);
-#if 0
-  typedef struct TS_req_st
-  {
-    ASN1_INTEGER *version;
-    TS_MSG_IMPRINT *msg_imprint;
-    ASN1_OBJECT *policy_id;   /* OPTIONAL */
-    ASN1_INTEGER *nonce;    /* OPTIONAL */
-    ASN1_BOOLEAN cert_req;    /* DEFAULT FALSE */
-    STACK_OF(X509_EXTENSION) *extensions; /* [0] OPTIONAL */
-  } TS_REQ;
-#endif
+
   lua_pushinteger(L, TS_REQ_get_version(req));
   lua_setfield(L, -2, "version");
 
@@ -451,11 +444,9 @@ static LUA_FUNCTION(openssl_ts_req_info)
     openssl_push_asn1object(L, TS_REQ_get_policy_id(req));
     lua_setfield(L, -2, "policy_id");
   }
-  if (TS_REQ_get_nonce(req))
-  {
-    PUSH_ASN1_INTEGER(L, TS_REQ_get_nonce(req));
-    lua_setfield(L, -2, "nonce");
-  }
+
+  openssl_push_asn1integer_as_bn(L, TS_REQ_get_nonce(req));
+  lua_setfield(L, -2, "nonce");
 
   lua_newtable(L);
   {
@@ -497,6 +488,7 @@ static luaL_Reg ts_req_funs[] =
   {"version",       openssl_ts_req_version},
   {"info",          openssl_ts_req_info},
   {"export",        openssl_ts_req_export},
+  {"add_ext",       openssl_ts_req_add_ext},
 
   {"to_verify_ctx", openssl_ts_req_to_verify_ctx},
 
@@ -551,33 +543,20 @@ static LUA_FUNCTION(openssl_ts_resp_export)
   return ret;
 }
 
-static int openssl_push_ts_accuracy(lua_State*L, const TS_ACCURACY* accuracy, int asobj)
+static int openssl_push_ts_accuracy(lua_State*L, const TS_ACCURACY* accuracy)
 {
   if (accuracy)
   {
-    if (asobj)
-    {
-      unsigned char *pbuf = NULL;
-      int len = i2d_TS_ACCURACY(accuracy, &pbuf);
-      if (len > 0)
-      {
-        lua_pushlstring(L, (const char*)pbuf, len);
-        OPENSSL_free(pbuf);
-      }
-      else
-        lua_pushnil(L);
-    }
-    else
-    {
-      lua_newtable(L);
+    lua_newtable(L);
 
-      PUSH_ASN1_INTEGER(L, TS_ACCURACY_get_micros(accuracy));
-      lua_setfield(L, -2, "micros");
-      PUSH_ASN1_INTEGER(L, TS_ACCURACY_get_millis(accuracy));
-      lua_setfield(L, -2, "millis");
-      PUSH_ASN1_INTEGER(L, TS_ACCURACY_get_seconds(accuracy));
-      lua_setfield(L, -2, "seconds");
-    }
+    openssl_push_asn1integer_as_bn(L, TS_ACCURACY_get_micros(accuracy));
+    lua_setfield(L, -2, "micros");
+
+    openssl_push_asn1integer_as_bn(L, TS_ACCURACY_get_millis(accuracy));
+    lua_setfield(L, -2, "millis");
+
+    openssl_push_asn1integer_as_bn(L, TS_ACCURACY_get_seconds(accuracy));
+    lua_setfield(L, -2, "seconds");
   }
   else
     lua_pushnil(L);
@@ -605,7 +584,7 @@ static int openssl_push_ts_msg_imprint(lua_State*L, TS_MSG_IMPRINT* imprint)
   return 1;
 };
 
-static int openssl_push_ts_tst_info(lua_State*L, TS_TST_INFO* info, const char* field)
+static int openssl_push_ts_tst_info(lua_State*L, TS_TST_INFO* info, const char* field, int asobj)
 {
   if(field==NULL)
   {
@@ -620,18 +599,18 @@ static int openssl_push_ts_tst_info(lua_State*L, TS_TST_INFO* info, const char*
     openssl_push_ts_msg_imprint(L, TS_TST_INFO_get_msg_imprint(info));
     lua_setfield(L, -2, "msg_imprint");
 
-    PUSH_ASN1_INTEGER(L, TS_TST_INFO_get_serial(info));
+    openssl_push_asn1integer_as_bn(L, TS_TST_INFO_get_serial(info));
     lua_setfield(L, -2, "serial");
 
     openssl_push_asn1(L, TS_TST_INFO_get_time(info), V_ASN1_GENERALIZEDTIME);
     lua_setfield(L, -2, "time");
 
-    openssl_push_ts_accuracy(L, TS_TST_INFO_get_accuracy(info), 1);
+    openssl_push_ts_accuracy(L, TS_TST_INFO_get_accuracy(info));
     lua_setfield(L, -2, "accuracy");
 
     AUXILIAR_SET(L, -1, "ordering", TS_TST_INFO_get_ordering(info), boolean);
 
-    PUSH_ASN1_INTEGER(L, TS_TST_INFO_get_nonce(info));
+    openssl_push_asn1integer_as_bn(L, TS_TST_INFO_get_nonce(info));
     lua_setfield(L, -2, "nonce");
 
     openssl_push_general_name(L, TS_TST_INFO_get_tsa(info));
@@ -660,7 +639,7 @@ static int openssl_push_ts_tst_info(lua_State*L, TS_TST_INFO* info, const char*
     }
     else if(strcmp(field, "serial")==0)
     {
-      PUSH_ASN1_INTEGER(L, TS_TST_INFO_get_serial(info));
+      openssl_push_asn1integer_as_bn(L, TS_TST_INFO_get_serial(info));
     }
     else if(strcmp(field, "time")==0)
     {
@@ -668,7 +647,7 @@ static int openssl_push_ts_tst_info(lua_State*L, TS_TST_INFO* info, const char*
     }
     else if(strcmp(field, "accuracy")==0)
     {
-      openssl_push_ts_accuracy(L, TS_TST_INFO_get_accuracy(info), 1);
+      openssl_push_ts_accuracy(L, TS_TST_INFO_get_accuracy(info));
     }
     else if(strcmp(field, "ordering")==0)
     {
@@ -676,7 +655,7 @@ static int openssl_push_ts_tst_info(lua_State*L, TS_TST_INFO* info, const char*
     }
     else if(strcmp(field, "nonce")==0)
     {
-      PUSH_ASN1_INTEGER(L, TS_TST_INFO_get_nonce(info));
+      openssl_push_asn1integer_as_bn(L, TS_TST_INFO_get_nonce(info));
     }
     else if(strcmp(field, "tsa")==0)
     {
@@ -708,11 +687,22 @@ static LUA_FUNCTION(openssl_ts_resp_tst_info)
 {
   TS_RESP *resp = CHECK_OBJECT(1, TS_RESP, "openssl.ts_resp");
   TS_TST_INFO *info = TS_RESP_get_tst_info(resp);
-  const char* field = lua_isnone(L, 2) ? NULL : luaL_checkstring(L, 2);
+  int typ = lua_type(L, 2);
+  int asobj = 0;
+  const char* field = NULL;
+  if (typ == LUA_TBOOLEAN)
+  {
+    asobj = lua_toboolean(L, 2);
+  }else if(typ == LUA_TSTRING)
+  {
+    field = luaL_checkstring(L, 2);
+    asobj = lua_toboolean(L, 3);
+  } else if(typ != LUA_TNONE)
+    luaL_argerror(L, 2, "must be boolean, string or none");
 
   if(info)
   {
-    if(openssl_push_ts_tst_info(L, info, field)==0)
+    if(openssl_push_ts_tst_info(L, info, field, asobj)==0)
       return luaL_argerror(L, 2, "invalid field of tst_info");
   }
   else
@@ -729,42 +719,47 @@ get info as table
 static LUA_FUNCTION(openssl_ts_resp_info)
 {
   TS_RESP *res = CHECK_OBJECT(1, TS_RESP, "openssl.ts_resp");
+  int asobj = lua_toboolean(L, 2);
 
   lua_newtable(L);
 
   {
     TS_STATUS_INFO *si = TS_RESP_get_status_info(res);
-    const ASN1_BIT_STRING *failure_info = TS_STATUS_INFO_get0_failure_info(si);
 
     lua_newtable(L);
-    PUSH_ASN1_INTEGER(L, TS_STATUS_INFO_get0_status(si));
+
+    openssl_push_asn1integer_as_bn(L, TS_STATUS_INFO_get0_status(si));
     lua_setfield(L, -2, "status");
 
-    if (failure_info)
     {
-      openssl_push_asn1(L, failure_info, V_ASN1_BIT_STRING);
-      lua_setfield(L, -2, "failure_info");
+      const ASN1_BIT_STRING *failure_info = TS_STATUS_INFO_get0_failure_info(si);
+      if (failure_info)
+      {
+        openssl_push_asn1(L, failure_info, V_ASN1_BIT_STRING);
+        lua_setfield(L, -2, "failure_info");
+      }
     }
 
-    if (TS_STATUS_INFO_get0_text(si))
     {
       const STACK_OF(ASN1_UTF8STRING) * sk = TS_STATUS_INFO_get0_text(si);
-      int i = 0, n = 0;
-      lua_newtable(L);
-      n = sk_ASN1_UTF8STRING_num(sk);
-      for (i = 0; i < n; i++)
+      if (sk)
       {
-        ASN1_UTF8STRING *x = sk_ASN1_UTF8STRING_value(sk, i);
-        lua_pushlstring(L, (const char*)x->data, x->length);
-        lua_rawseti(L, -2, i + 1);
+        int i = 0, n = 0;
+        lua_newtable(L);
+        n = sk_ASN1_UTF8STRING_num(sk);
+        for (i = 0; i < n; i++)
+        {
+          ASN1_UTF8STRING *x = sk_ASN1_UTF8STRING_value(sk, i);
+          lua_pushlstring(L, (const char*)x->data, x->length);
+          lua_rawseti(L, -2, i + 1);
+        }
+        lua_setfield(L, -2, "text");
       }
-      lua_setfield(L, -2, "text");
     }
 
     lua_setfield(L, -2, "status_info");
   }
 
-
   if (TS_RESP_get_token(res))
   {
     PKCS7* token = PKCS7_dup(TS_RESP_get_token(res));
@@ -773,7 +768,7 @@ static LUA_FUNCTION(openssl_ts_resp_info)
 
   if (TS_RESP_get_tst_info(res))
   {
-    openssl_push_ts_tst_info(L, TS_RESP_get_tst_info(res), NULL);
+    openssl_push_ts_tst_info(L, TS_RESP_get_tst_info(res), NULL, asobj);
     lua_setfield(L, -2, "tst_info");
   }
 
@@ -839,7 +834,7 @@ set signer cert and pkey
 @tparam evp_pkey pkey signer pkey
 @treturn boolean result
 */
-static LUA_FUNCTION(openssl_ts_resp_ctx_singer)
+static LUA_FUNCTION(openssl_ts_resp_ctx_signer)
 {
   TS_RESP_CTX *ctx = CHECK_OBJECT(1, TS_RESP_CTX, "openssl.ts_resp_ctx");
   X509 *signer = CHECK_OBJECT(2, X509, "openssl.x509");
@@ -976,57 +971,11 @@ static LUA_FUNCTION(openssl_ts_resp_ctx_clock_precision_digits)
   return openssl_pushresult(L, ret);
 }
 
-/***
-set status info
-@function set_status_info
-@tparam integer status
-@tparam string text
-@treturn boolean result
-*/
-static LUA_FUNCTION(openssl_ts_resp_ctx_set_status_info)
-{
-  TS_RESP_CTX *ctx = CHECK_OBJECT(1, TS_RESP_CTX, "openssl.ts_resp_ctx");
-  int status = luaL_checkint(L, 2);
-  const char* text = luaL_checkstring(L, 3);
-  int ret = TS_RESP_CTX_set_status_info(ctx, status, text);
-  return openssl_pushresult(L, ret);
-}
-
-/***
-set status info cond
-@function set_status_info_cond
-@tparam integer status
-@tparam string text
-@treturn boolean result
-*/
-static LUA_FUNCTION(openssl_ts_resp_ctx_set_status_info_cond)
-{
-  TS_RESP_CTX *ctx = CHECK_OBJECT(1, TS_RESP_CTX, "openssl.ts_resp_ctx");
-  int status = luaL_checkint(L, 2);
-  const char* text = luaL_checkstring(L, 3);
-  int ret = TS_RESP_CTX_set_status_info_cond(ctx, status, text);
-  return openssl_pushresult(L, ret);
-}
-
-/***
-add failure info
-@function add_failure_info
-@tparam integer failure
-@treturn result
-*/
-static LUA_FUNCTION(openssl_ts_resp_ctx_add_failure_info)
-{
-  TS_RESP_CTX *ctx = CHECK_OBJECT(1, TS_RESP_CTX, "openssl.ts_resp_ctx");
-  int failure = luaL_checkint(L, 2);
-  int ret = TS_RESP_CTX_add_failure_info(ctx, failure);
-  return openssl_pushresult(L, ret);
-}
-
 /***
 get flags
 @function flags
 */
-static LUA_FUNCTION(openssl_ts_resp_ctx_flags)
+static LUA_FUNCTION(openssl_ts_resp_ctx_add_flags)
 {
   TS_RESP_CTX *ctx = CHECK_OBJECT(1, TS_RESP_CTX, "openssl.ts_resp_ctx");
   int flags = luaL_checkint(L, 2);
@@ -1091,11 +1040,22 @@ static LUA_FUNCTION(openssl_ts_resp_ctx_tst_info)
 {
   TS_RESP_CTX *ctx = CHECK_OBJECT(1, TS_RESP_CTX, "openssl.ts_resp_ctx");
   TS_TST_INFO *info = TS_RESP_CTX_get_tst_info(ctx);
-  const char* field = lua_isnone(L, 2) ? NULL : luaL_checkstring(L, 2);
+  int typ = lua_type(L, 2);
+  int asobj = 0;
+  const char* field = NULL;
+  if (typ == LUA_TBOOLEAN)
+  {
+    asobj = lua_toboolean(L, 2);
+  }else if(typ == LUA_TSTRING)
+  {
+    field = luaL_checkstring(L, 2);
+    asobj = lua_toboolean(L, 3);
+  } else if(typ != LUA_TNONE)
+    luaL_argerror(L, 2, "must be boolean, string or none");
 
   if(info)
   {
-    if(openssl_push_ts_tst_info(L, info, field)==0)
+    if(openssl_push_ts_tst_info(L, info, field, asobj)==0)
       return luaL_argerror(L, 2, "invalid field of tst_info");
   }
   else
@@ -1322,21 +1282,18 @@ static LUA_FUNCTION(openssl_ts_resp_ctx_gc)
 static luaL_Reg ts_resp_ctx_funs[] =
 {
   /* get and set */
-  {"signer",              openssl_ts_resp_ctx_singer},
+  {"signer",              openssl_ts_resp_ctx_signer},
   {"certs",               openssl_ts_resp_ctx_certs},
   {"default_policy",      openssl_ts_resp_ctx_default_policy},
   {"policies",            openssl_ts_resp_ctx_policies},
   {"accuracy",            openssl_ts_resp_ctx_accuracy},
   {"clock_precision_digits",  openssl_ts_resp_ctx_clock_precision_digits},
   {"md",                  openssl_ts_resp_ctx_md},
-  {"flags",               openssl_ts_resp_ctx_flags},
+  {"add_flags",           openssl_ts_resp_ctx_add_flags},
 
   /* set */
-  {"set_status_info",         openssl_ts_resp_ctx_set_status_info},
-  {"set_status_info_cond",    openssl_ts_resp_ctx_set_status_info_cond},
   {"set_serial_cb",      openssl_ts_resp_ctx_set_serial_cb},
   {"set_time_cb",        openssl_ts_resp_ctx_set_time_cb},
-  {"add_failure_info",   openssl_ts_resp_ctx_add_failure_info},
 
   /* get */
   {"request",            openssl_ts_resp_ctx_request},
@@ -1510,6 +1467,39 @@ static luaL_Reg ts_verify_ctx_funs[] =
 };
 #endif
 
+static LuaL_Enumeration ots_const[] =
+{
+  {"STATUS_GRANTED",                 TS_STATUS_GRANTED},
+  {"STATUS_GRANTED_WITH_MODS",       TS_STATUS_GRANTED_WITH_MODS},
+  {"STATUS_REJECTION",               TS_STATUS_REJECTION},
+  {"STATUS_WAITING",                 TS_STATUS_WAITING},
+  {"STATUS_REVOCATION_WARNIN",       TS_STATUS_REVOCATION_WARNING},
+  {"STATUS_REVOCATION_NOTIFICATION", TS_STATUS_REVOCATION_NOTIFICATION},
+
+  {"INFO_BAD_ALG",               TS_INFO_BAD_ALG},
+  {"INFO_BAD_REQUEST",           TS_INFO_BAD_REQUEST},
+  {"INFO_BAD_DATA_FORMA",        TS_INFO_BAD_DATA_FORMAT},
+  {"INFO_TIME_NOT_AVAILABL",     TS_INFO_TIME_NOT_AVAILABLE},
+  {"INFO_UNACCEPTED_POLICY",     TS_INFO_UNACCEPTED_POLICY},
+  {"INFO_UNACCEPTED_EXTENSION",  TS_INFO_UNACCEPTED_EXTENSION},
+  {"INFO_ADD_INFO_NOT_AVAILABL", TS_INFO_ADD_INFO_NOT_AVAILABLE},
+  {"INFO_SYSTEM_FAILUR",         TS_INFO_SYSTEM_FAILURE},
+
+  {"VFY_SIGNATURE",   TS_VFY_SIGNATURE},
+  {"VFY_VERSION",     TS_VFY_VERSION},
+  {"VFY_POLICY",      TS_VFY_POLICY},
+  {"VFY_IMPRINT",     TS_VFY_IMPRINT},
+  {"VFY_DATA",        TS_VFY_DATA},
+  {"VFY_NONCE",       TS_VFY_NONCE},
+  {"VFY_SIGNER",      TS_VFY_SIGNER},
+  {"VFY_TSA_NAME",    TS_VFY_TSA_NAME},
+  {"VFY_ALL_IMPRINT", TS_VFY_ALL_IMPRINT},
+  {"VFY_ALL_DATA",    TS_VFY_ALL_DATA},
+
+
+  {NULL,           0}
+};
+
 int luaopen_ts(lua_State *L)
 {
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
@@ -1520,6 +1510,8 @@ int luaopen_ts(lua_State *L)
 
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
+
+  auxiliar_enumerate(L, -1, ots_const);
 #else
   lua_pushnil(L);
 #endif
