@@ -19,31 +19,34 @@ create ssl_ctx object, which mapping to SSL_CTX in openssl.
 @tparam[opt] string support_ciphers, if not given, default of openssl will be used
 @treturn ssl_ctx
 */
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
 #define  TLS_PROTOCOL_TIPS  \
   "only support TLS, DTLS to negotiate highest available SSL/TLS or DTLS " \
   "version above openssl v1.1.0\n" \
   "optional followed by _client or _server\n" \
   "default is TLS\n"
+#define DEFAULT_PROTOCOL "TLS"
 #else
 #define  TLS_PROTOCOL_TIPS  \
   "SSLv23, TLSv1_2, TLSv1_1, TLSv1, DTLSv1_2 or DTLSv1, optional followed by _client or _server\n" \
   "optional followed by _client or _server\n" \
   "default is SSLv23 to negotiate highest available SSL/TLS\n"
+#define DEFAULT_PROTOCOL "SSLv23"
 #endif
 
-typedef struct
-{
-  int session_add;
-  int session_get;
-  int session_del;
+typedef enum{
+  SSL_CTX_SESSION_ADD = 0,
+  SSL_CTX_SESSION_GET,
+  SSL_CTX_SESSION_DEL,
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-  int tmp_dh;
-  int tmp_rsa;
-  int tmp_ecdh;
-  int tmp_ec_curve;
+
+  SSL_CTX_TEMP_DH,
+  SSL_CTX_TEMP_RSA,
+  SSL_CTX_TEMP_ECDH,
+  SSL_CTX_TEMP_EC_CURVE,
 #endif
-} openssl_ssl_ctx_lua;
+  SSL_CTX_MAX_IDX
+}SSL_CTX_INDEX;
 
 #if OPENSSL_VERSION_NUMBER < 0x10002000L
 static int SSL_is_server(const SSL *s)
@@ -54,11 +57,7 @@ static int SSL_is_server(const SSL *s)
 
 static int openssl_ssl_ctx_new(lua_State*L)
 {
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
-  const char* meth = luaL_optstring(L, 1, "TLS");
-#else
-  const char* meth = luaL_optstring(L, 1, "SSLv23");
-#endif
+  const char* meth = luaL_optstring(L, 1, DEFAULT_PROTOCOL);
 #if OPENSSL_VERSION_NUMBER >= 0x01000000L
   const
 #endif
@@ -67,7 +66,7 @@ static int openssl_ssl_ctx_new(lua_State*L)
   SSL_CTX* ctx;
 
   if (0);
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
   else if (strcmp(meth, "TLS") == 0)
     method = TLS_method();
   else if (strcmp(meth, "TLS_server") == 0)
@@ -163,32 +162,48 @@ static int openssl_ssl_ctx_new(lua_State*L)
 }
 
 /***
-get alert_string for ssl state
-@function alert_string
+get alert_type for ssl state
+@function alert_type
 @tparam number alert
 @tparam[opt=false] boolean long
 @treturn string alert type
-@treturn string desc string, if long set true will return long info
 */
-static int openssl_ssl_alert_string(lua_State*L)
+static int openssl_ssl_alert_type(lua_State*L)
 {
   int v = luaL_checkint(L, 1);
   int _long = lua_isnone(L, 2) ? 0 : auxiliar_checkboolean(L, 2);
   const char* val;
 
   if (_long)
-    val = SSL_alert_type_string_long(v);
+    val = SSL_alert_type_string_long(v << 8);
   else
-    val = SSL_alert_type_string(v);
+    val = SSL_alert_type_string(v << 8);
   lua_pushstring(L, val);
 
+  return 1;
+}
+
+/***
+get alert_desc for ssl state
+@function alert_desc
+@tparam number alert
+@tparam[opt=false] boolean long
+@treturn string alert type
+@treturn string desc string, if long set true will return long info
+*/
+static int openssl_ssl_alert_desc(lua_State*L)
+{
+  int v = luaL_checkint(L, 1);
+  int _long = lua_isnone(L, 2) ? 0 : auxiliar_checkboolean(L, 2);
+  const char* val;
+
   if (_long)
     val = SSL_alert_desc_string_long(v);
   else
     val = SSL_alert_desc_string(v);
   lua_pushstring(L, val);
 
-  return 2;
+  return 1;
 }
 
 static int openssl_ssl_session_new(lua_State*L)
@@ -219,7 +234,8 @@ static int openssl_ssl_session_read(lua_State*L)
 static luaL_Reg R[] =
 {
   {"ctx_new",       openssl_ssl_ctx_new },
-  {"alert_string",  openssl_ssl_alert_string },
+  {"alert_type",    openssl_ssl_alert_type },
+  {"alert_desc",    openssl_ssl_alert_desc },
 
   {"session_new",   openssl_ssl_session_new},
   {"session_read",  openssl_ssl_session_read},
@@ -551,7 +567,15 @@ static int openssl_ssl_ctx_load_verify_locations(lua_State*L)
   SSL_CTX* ctx = CHECK_OBJECT(1, SSL_CTX, "openssl.ssl_ctx");
   const char* CAfile = luaL_checkstring(L, 2);
   const char* CApath = luaL_optstring(L, 3, NULL);
+#if (OPENSSL_VERSION_NUMBER >= 0x30000000L)
+  int ret = !(CAfile == NULL && CApath == NULL);
+  if (CAfile != NULL)
+    ret = SSL_CTX_load_verify_file(ctx, CAfile);
+  if ( ret==1 && CApath != NULL)
+    ret = SSL_CTX_load_verify_dir(ctx, CApath);
+#else
   int ret = SSL_CTX_load_verify_locations(ctx, CAfile, CApath);
+#endif
   return openssl_pushresult(L, ret);
 }
 
@@ -893,61 +917,49 @@ static int openssl_ssl_ctx_set_cert_verify(lua_State*L)
   return 0;
 }
 
-static openssl_ssl_ctx_lua* openssl_get_ssl_ctx_lua(lua_State*L, SSL_CTX *sctx)
-{
-  openssl_ssl_ctx_lua *ctx = NULL;
-  openssl_valueget(L, sctx, "refvalue");
-  if (lua_isnil(L, -1))
-  {
-    ctx = (openssl_ssl_ctx_lua*)lua_newuserdata(L, sizeof(openssl_ssl_ctx_lua));
-    ctx->session_add = ctx->session_del = ctx->session_get = LUA_NOREF;
-    openssl_valueset(L, sctx, "refvalue");
-  }
-  else
-    ctx = (openssl_ssl_ctx_lua*)lua_touserdata(L, -1);
-  lua_pop(L, 1);
-  return ctx;
-}
-
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
 static DH *tmp_dh_callback(SSL *ssl, int is_export, int keylength)
 {
   DH *dh_tmp = NULL;
   SSL_CTX *sctx = SSL_get_SSL_CTX(ssl);
   lua_State *L = SSL_CTX_get_app_data(sctx);
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
-  int ret = 0;
-  /* get callback function */
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->tmp_dh);
 
-  /* Invoke the callback */
-  lua_pushboolean(L, is_export);
-  lua_pushnumber(L, keylength);
-  ret = lua_pcall(L, 2, 1, 0);
-  if (ret == 0)
+  int type = openssl_valuegeti(L, sctx, SSL_CTX_TEMP_DH);
+  if (type == LUA_TFUNCTION)
   {
-    BIO *bio;
-    /* Load parameters from returned value */
-    if (lua_type(L, -1) != LUA_TSTRING)
+    int ret;
+    /* top is callback function */
+    /* Invoke the callback */
+    lua_pushboolean(L, is_export);
+    lua_pushnumber(L, keylength);
+    ret = lua_pcall(L, 2, 1, 0);
+    if (ret == 0)
     {
-      lua_pop(L, 2);  /* Remove values from stack */
-      return NULL;
+      BIO *bio;
+      /* Load parameters from returned value */
+      if (lua_type(L, -1) != LUA_TSTRING)
+      {
+        lua_pop(L, 2);  /* Remove values from stack */
+        return NULL;
+      }
+      bio = BIO_new_mem_buf((void*)lua_tostring(L, -1),
+                            lua_rawlen(L, -1));
+      if (bio)
+      {
+        dh_tmp = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
+        BIO_free(bio);
+      }
     }
-    bio = BIO_new_mem_buf((void*)lua_tostring(L, -1),
-                          lua_rawlen(L, -1));
-    if (bio)
+    else
     {
-      dh_tmp = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
-      BIO_free(bio);
+      lua_error(L);
     }
-  }
-  else
-  {
-    lua_error(L);
-  }
 
-  lua_pop(L, 2);    /* Remove values from stack */
-  return dh_tmp;
+    lua_pop(L, 2);    /* Remove values from stack */
+    return dh_tmp;
+  }
+  lua_pop(L, 1);
+  return NULL;
 }
 
 static RSA *tmp_rsa_callback(SSL *ssl, int is_export, int keylength)
@@ -955,38 +967,41 @@ static RSA *tmp_rsa_callback(SSL *ssl, int is_export, int keylength)
   RSA *rsa_tmp = NULL;
   SSL_CTX *sctx = SSL_get_SSL_CTX(ssl);
   lua_State *L = SSL_CTX_get_app_data(sctx);
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
-  int ret = 0;
-  /* get callback function */
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->tmp_rsa);
-
-  /* Invoke the callback */
-  lua_pushboolean(L, is_export);
-  lua_pushnumber(L, keylength);
-  ret = lua_pcall(L, 2, 1, 0);
-  if (ret == 0)
+  int type = openssl_valuegeti(L, sctx, SSL_CTX_TEMP_RSA);
+  if (type == LUA_TFUNCTION)
   {
-    BIO *bio;
-    /* Load parameters from returned value */
-    if (lua_type(L, -1) != LUA_TSTRING)
+    int ret;
+    /* top is callback function */
+    /* Invoke the callback */
+    lua_pushboolean(L, is_export);
+    lua_pushnumber(L, keylength);
+    ret = lua_pcall(L, 2, 1, 0);
+    if (ret == 0)
     {
-      lua_pop(L, 2);  /* Remove values from stack */
-      return NULL;
+      BIO *bio;
+      /* Load parameters from returned value */
+      if (lua_type(L, -1) != LUA_TSTRING)
+      {
+        lua_pop(L, 2);  /* Remove values from stack */
+        return NULL;
+      }
+      bio = BIO_new_mem_buf((void*)lua_tostring(L, -1), lua_rawlen(L, -1));
+      if (bio)
+      {
+        rsa_tmp = PEM_read_bio_RSAPrivateKey(bio, NULL, NULL, NULL);
+        BIO_free(bio);
+      }
     }
-    bio = BIO_new_mem_buf((void*)lua_tostring(L, -1), lua_rawlen(L, -1));
-    if (bio)
+    else
     {
-      rsa_tmp = PEM_read_bio_RSAPrivateKey(bio, NULL, NULL, NULL);
-      BIO_free(bio);
+      lua_error(L);
     }
-  }
-  else
-  {
-    lua_error(L);
-  }
 
-  lua_pop(L, 2);    /* Remove values from stack */
-  return rsa_tmp;
+    lua_pop(L, 2);    /* Remove values from stack */
+    return rsa_tmp;
+  }
+  lua_pop(L, 1);
+  return NULL;
 }
 
 static EC_KEY *tmp_ecdh_callback(SSL *ssl, int is_export, int keylength)
@@ -994,39 +1009,42 @@ static EC_KEY *tmp_ecdh_callback(SSL *ssl, int is_export, int keylength)
   EC_KEY *ec_tmp = NULL;
   SSL_CTX *sctx = SSL_get_SSL_CTX(ssl);
   lua_State *L = SSL_CTX_get_app_data(sctx);
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
-  int ret = 0;
-  /* get callback function */
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->tmp_ecdh);
-
-  /* Invoke the callback */
-  lua_pushboolean(L, is_export);
-  lua_pushnumber(L, keylength);
-  ret = lua_pcall(L, 2, 1, 0);
-  if (ret == 0)
+  int type = openssl_valuegeti(L, sctx, SSL_CTX_TEMP_ECDH);
+  if (type == LUA_TFUNCTION)
   {
-    BIO *bio;
-    /* Load parameters from returned value */
-    if (lua_type(L, -1) != LUA_TSTRING)
+    int ret;
+    /* top is callback function */
+    /* Invoke the callback */
+    lua_pushboolean(L, is_export);
+    lua_pushnumber(L, keylength);
+    ret = lua_pcall(L, 2, 1, 0);
+    if (ret == 0)
     {
-      lua_pop(L, 2);  /* Remove values from stack */
-      return NULL;
+      BIO *bio;
+      /* Load parameters from returned value */
+      if (lua_type(L, -1) != LUA_TSTRING)
+      {
+        lua_pop(L, 2);  /* Remove values from stack */
+        return NULL;
+      }
+      bio = BIO_new_mem_buf((void*)lua_tostring(L, -1),
+                            lua_rawlen(L, -1));
+      if (bio)
+      {
+        ec_tmp = PEM_read_bio_ECPrivateKey(bio, NULL, NULL, NULL);
+        BIO_free(bio);
+      }
     }
-    bio = BIO_new_mem_buf((void*)lua_tostring(L, -1),
-                          lua_rawlen(L, -1));
-    if (bio)
+    else
     {
-      ec_tmp = PEM_read_bio_ECPrivateKey(bio, NULL, NULL, NULL);
-      BIO_free(bio);
+      lua_error(L);
     }
-  }
-  else
-  {
-    lua_error(L);
-  }
 
-  lua_pop(L, 2);    /* Remove values from stack */
-  return ec_tmp;
+    lua_pop(L, 2);    /* Remove values from stack */
+    return ec_tmp;
+  }
+  lua_pop(L, 1);
+  return NULL;
 }
 
 /***
@@ -1051,7 +1069,6 @@ set ecdh with given curvename as tmp key
 static int openssl_ssl_ctx_set_tmp(lua_State *L)
 {
   SSL_CTX* ctx = CHECK_OBJECT(1, SSL_CTX, "openssl.ssl_ctx");
-  openssl_ssl_ctx_lua *lctx = openssl_get_ssl_ctx_lua(L, ctx);
   static const char* which[] =
   {
     "dh",
@@ -1070,20 +1087,20 @@ static int openssl_ssl_ctx_set_tmp(lua_State *L)
     switch (nwhich)
     {
     case 0:
-      lctx->tmp_dh = luaL_ref(L, LUA_REGISTRYINDEX);
+      openssl_valueseti(L, ctx, SSL_CTX_TEMP_DH);
       SSL_CTX_set_tmp_dh_callback(ctx, tmp_dh_callback);
       break;
     case 1:
-      lctx->tmp_rsa = luaL_ref(L, LUA_REGISTRYINDEX);
+      openssl_valueseti(L, ctx, SSL_CTX_TEMP_RSA);
       SSL_CTX_set_tmp_rsa_callback(ctx, tmp_rsa_callback);
       break;
     case 2:
     {
+      openssl_valueseti(L, ctx, SSL_CTX_TEMP_ECDH);
       luaL_argcheck(L, lua_isstring(L, 4), 4, "must supply curve name");
-      lctx->tmp_ecdh = luaL_ref(L, LUA_REGISTRYINDEX);
-      SSL_CTX_set_tmp_ecdh_callback(ctx, tmp_ecdh_callback);
       lua_pushvalue(L, 4);
-      lctx->tmp_ec_curve = luaL_ref(L, LUA_REGISTRYINDEX);
+      openssl_valueseti(L, ctx, SSL_CTX_TEMP_EC_CURVE);
+      SSL_CTX_set_tmp_ecdh_callback(ctx, tmp_ecdh_callback);
     }
     break;
     }
@@ -1199,18 +1216,17 @@ static int openssl_add_session(SSL *ssl, SSL_SESSION *session)
   int ret;
   SSL_CTX *sctx = SSL_get_SSL_CTX(ssl);
   lua_State *L = SSL_CTX_get_app_data(sctx);
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
 
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->session_add);
-  openssl_refrence(L, ssl, 1);
+  openssl_valuegeti(L, sctx, SSL_CTX_SESSION_ADD);
+  SSL_up_ref(ssl);
   PUSH_OBJECT(ssl, "openssl.ssl");
-  openssl_refrence(L, session, 1);
+  SSL_SESSION_up_ref(session);
   PUSH_OBJECT(session, "openssl.ssl_session");
 
   ret = lua_pcall(L, 2, 1, 0);
   if (ret != LUA_OK)
   {
-    fprintf(stderr, "Uncatch error: %s\n", lua_tostring(L, -1));
+    fprintf(stderr, "add session callback error: %s\n", lua_tostring(L, -1));
     ret = 0;
   }
   else
@@ -1226,18 +1242,17 @@ static SSL_SESSION *openssl_get_session(SSL *ssl,
   int ret;
   SSL_CTX *sctx = SSL_get_SSL_CTX(ssl);
   lua_State *L = SSL_CTX_get_app_data(sctx);
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
   SSL_SESSION *session = NULL;
 
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->session_get);
-  openssl_refrence(L, ssl, 1);
+  openssl_valuegeti(L, sctx, SSL_CTX_SESSION_GET);
+  SSL_up_ref(ssl);
   PUSH_OBJECT(ssl, "openssl.ssl");
   lua_pushlstring(L, (const char*)id, idlen);
 
   ret = lua_pcall(L, 2, 1, 0);
   if (ret != LUA_OK)
   {
-    fprintf(stderr, "Uncatch error: %s\n", lua_tostring(L, -1));
+    fprintf(stderr, "get session callback error: %s\n", lua_tostring(L, -1));
     lua_pop(L, 1);
     return NULL;
   }
@@ -1262,18 +1277,18 @@ static void openssl_del_session(SSL_CTX *sctx, SSL_SESSION *session)
 {
   int ret;
   lua_State *L = SSL_CTX_get_app_data(sctx);
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
 
-  lua_rawgeti(L, LUA_REGISTRYINDEX, ctx->session_del);
+  openssl_valuegeti(L, sctx, SSL_CTX_SESSION_DEL);
+
   openssl_refrence(L, sctx, 1);
   PUSH_OBJECT(sctx, "openssl.ssl_ctx");
-  openssl_refrence(L, session, 1);
+  SSL_SESSION_up_ref( session);
   PUSH_OBJECT(session, "openssl.ssl_session");
 
   ret = lua_pcall(L, 2, 0, 0);
   if (ret != LUA_OK)
   {
-    fprintf(stderr, "Uncatch error: %s\n", lua_tostring(L, -1));
+    fprintf(stderr, "del session callback error: %s\n", lua_tostring(L, -1));
     lua_pop(L, 1);
   }
 }
@@ -1281,29 +1296,29 @@ static void openssl_del_session(SSL_CTX *sctx, SSL_SESSION *session)
 static int openssl_ssl_ctx_set_session_callback(lua_State*L)
 {
   SSL_CTX* sctx = CHECK_OBJECT(1, SSL_CTX, "openssl.ssl_ctx");
-  openssl_ssl_ctx_lua *ctx = openssl_get_ssl_ctx_lua(L, sctx);
   if (!lua_isnoneornil(L, 2))
   {
     luaL_checktype(L, 2, LUA_TFUNCTION);
     lua_pushvalue(L, 2);
-    ctx->session_add = luaL_ref(L, LUA_REGISTRYINDEX);
+    openssl_valueseti(L, sctx, SSL_CTX_SESSION_ADD);
     SSL_CTX_sess_set_new_cb(sctx, openssl_add_session);
   }
   if (!lua_isnoneornil(L, 3))
   {
     luaL_checktype(L, 3, LUA_TFUNCTION);
     lua_pushvalue(L, 3);
-    ctx->session_get = luaL_ref(L, LUA_REGISTRYINDEX);
+    openssl_valueseti(L, sctx, SSL_CTX_SESSION_GET);
     SSL_CTX_sess_set_get_cb(sctx, openssl_get_session);
   }
   if (!lua_isnoneornil(L, 4))
   {
     luaL_checktype(L, 4, LUA_TFUNCTION);
     lua_pushvalue(L, 4);
-    ctx->session_del = luaL_ref(L, LUA_REGISTRYINDEX);
+    openssl_valueseti(L, sctx, SSL_CTX_SESSION_DEL);
     SSL_CTX_sess_set_remove_cb(sctx, openssl_del_session);
   }
-  return 0;
+  lua_pushvalue(L, 1);
+  return 1;
 }
 
 /***
@@ -1362,7 +1377,7 @@ set session cache mode,and return old mode
 @tparam string mode support 'no_auto_clear','server','client','both','off',
 'no_auto_clear' can be combine with others, so accept one or two param.
 */
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
 #endif
 static int openssl_session_cache_mode(lua_State *L)
 {
@@ -1596,7 +1611,7 @@ static int openssl_ssl_session_peer(lua_State*L)
 {
   SSL_SESSION* session = CHECK_OBJECT(1, SSL_SESSION, "openssl.ssl_session");
   X509 *x = SSL_SESSION_get0_peer(session);
-  PUSH_OBJECT(x, "openssl.x509");
+  PUSH_OBJECT(X509_dup(x), "openssl.x509");
   return 1;
 }
 #endif
@@ -1615,7 +1630,7 @@ static int openssl_ssl_session_id(lua_State*L)
   }
   else
   {
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
     size_t len;
     const char* id = luaL_checklstring(L, 2, &len);
     int ret = SSL_SESSION_set1_id((SSL_SESSION*)session, (const unsigned char*)id, len);
@@ -1668,7 +1683,7 @@ static int openssl_ssl_session_is_resumable(lua_State*L)
 }
 #endif
 
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
 static int openssl_ssl_session_has_ticket(lua_State*L)
 {
   SSL_SESSION* session = CHECK_OBJECT(1, SSL_SESSION, "openssl.ssl_session");
@@ -1691,7 +1706,7 @@ static luaL_Reg ssl_session_funcs[] =
 #if OPENSSL_VERSION_NUMBER > 0x10101000L && !defined(LIBRESSL_VERSION_NUMBER)
   {"is_resumable",  openssl_ssl_session_is_resumable},
 #endif
-#if OPENSSL_VERSION_NUMBER > 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER > 0x10100000L
   {"has_ticket",    openssl_ssl_session_has_ticket},
 #endif
 
@@ -1806,7 +1821,7 @@ static int openssl_ssl_want(lua_State*L)
   lua_pushinteger(L, st);
   return 2;
 }
-#if !defined(OPENSSL_NO_COMP) && !defined(LIBRESSL_VERSION_NUMBER)
+#if !defined(OPENSSL_NO_COMP)
 /***
 get current compression name
 @function current_compression
@@ -1836,7 +1851,7 @@ static int openssl_ssl_current_cipher(lua_State *L)
   if (c)
   {
     int bits, algbits;
-    char err[LUAL_BUFFERSIZE] = {0};;
+    char err[LUAL_BUFFERSIZE] = {0};
 
     lua_newtable(L);
 
@@ -1876,44 +1891,45 @@ static int openssl_ssl_pushresult(lua_State* L, SSL*ssl, int ret_code)
   switch (err)
   {
   case SSL_ERROR_NONE:
+    lua_pushboolean(L, 1);
     lua_pushinteger(L, ret_code);
-    return 1;
+    break;
   case SSL_ERROR_ZERO_RETURN:
     lua_pushboolean(L, 0);
     lua_pushinteger(L, 0);
-    return 2;
+    break;
   case SSL_ERROR_SSL:
     lua_pushnil(L);
     lua_pushstring(L, "ssl");
-    return 2;
+    break;
   case SSL_ERROR_WANT_READ:
     lua_pushboolean(L, 0);
     lua_pushstring(L, "want_read");
-    return 2;
+    break;
   case SSL_ERROR_WANT_WRITE:
     lua_pushboolean(L, 0);
     lua_pushstring(L, "want_write");
-    return 2;
+    break;
   case SSL_ERROR_WANT_X509_LOOKUP:
     lua_pushboolean(L, 0);
     lua_pushstring(L, "want_x509_lookup");
-    return 2;
+    break;
   case SSL_ERROR_SYSCALL:
     lua_pushnil(L);
     lua_pushstring(L, "syscall");
-    return 2;
+    break;
   case SSL_ERROR_WANT_CONNECT:
     lua_pushboolean(L, 0);
     lua_pushstring(L, "want_connect");
-    return 2;
+    break;
   case SSL_ERROR_WANT_ACCEPT:
     lua_pushboolean(L, 0);
     lua_pushstring(L, "want_accept");
-    return 2;
+    break;
   default:
-    lua_pushnil(L);
-    return 1;
+    return 0;
   }
+  return 2;
 }
 
 /***
@@ -1961,9 +1977,10 @@ get value according to arg
  <br/>iversion
  <br/>version
  <br/>default_timeout,
- <br/>certificates
+ <br/>certificate
  <br/>verify_result
  <br/>state
+ <br/>hostname
  <br/>state_string
  <br/>side
 @return according to arg
@@ -2047,17 +2064,16 @@ static int openssl_ssl_get(lua_State*L)
     else if (strcmp(what, "certificate") == 0)
     {
       X509* cert = SSL_get_certificate(s);
-      PUSH_OBJECT(cert, "openssl.x509");
+      if (cert)
+        PUSH_OBJECT(X509_dup(cert), "openssl.x509");
+      else
+        lua_pushnil(L);
     }
     else if (strcmp(what, "verify_result") == 0)
     {
       long l = SSL_get_verify_result(s);
       lua_pushinteger(L, l);
     }
-    else if (strcmp(what, "version") == 0)
-    {
-      lua_pushstring(L, SSL_get_version(s));
-    }
     else if (strcmp(what, "state") == 0)
     {
       lua_pushinteger(L, SSL_get_state(s));
@@ -2562,7 +2578,7 @@ static luaL_Reg ssl_funcs[] =
   {"is_server", openssl_ssl_is_server},
 
   {"current_cipher",        openssl_ssl_current_cipher},
-#if !defined(OPENSSL_NO_COMP) && !defined(LIBRESSL_VERSION_NUMBER)
+#if !defined(OPENSSL_NO_COMP)
   {"current_compression",   openssl_ssl_current_compression},
 #endif
   {"getpeerverification",   openssl_ssl_getpeerverification},
@@ -2621,6 +2637,8 @@ int luaopen_ssl(lua_State *L)
     lua_pushinteger(L, iVerifyMode_Options[i]);
     lua_setfield(L, -2, sVerifyMode_Options[i]);
   }
+  lua_pushstring(L, DEFAULT_PROTOCOL);
+  lua_setfield(L, -2, "default");
 
   return 1;
 }
